"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.compressImagesFactory = void 0;
const child_process_1 = __importDefault(require("child_process"));
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const cloneDeep_1 = __importDefault(require("lodash/cloneDeep"));
const constants_1 = require("../constants");
const utils_1 = require("./utils");
const index_1 = require("./index");
const errorReporting_1 = require("./errorReporting");
let jpegRecompressBinName;
switch (process.platform) {
    case 'win32':
        jpegRecompressBinName = 'jpeg-recompress.exe';
        break;
    default:
        jpegRecompressBinName = 'jpeg-recompress';
        break;
}
const jpegRecompressBin = path_1.default.join(__dirname, '..', '..', 'vendor', process.platform, jpegRecompressBinName);
if (!fs_extra_1.default.existsSync(jpegRecompressBin)) {
    (0, errorReporting_1.reportNoBinFound)(jpegRecompressBin);
}
function mergeImageData(imageData, newFields) {
    return Object.assign(Object.assign({}, imageData), newFields);
}
/**
 * Takes a list of md5 hashed ids for images that should be compressed and compress them one at a time
 * Each time an image is compressed or fails, will emit an IPC event with the file md5 and the status (success/fail)
 * It will also create a backup of the original file before compressing the image
 *
 * @param imageIds
 */
function compressImagesFactory(serviceContainer, imageDataStore, runtimeStore) {
    return function (siteID, imageMD5s, stripMetaData) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                (0, errorReporting_1.reportCompressRequest)(siteID);
                const site = serviceContainer.siteData.getSite(siteID);
                if (!site) {
                    return;
                }
                const backupDirPath = path_1.default.join(site.longPath, constants_1.BACKUP_DIR_NAME);
                fs_extra_1.default.ensureDir(backupDirPath);
                /**
                 * deep clone this so that the existing store does not get corrupted until we explicity update it
                 */
                const siteData = (0, cloneDeep_1.default)(imageDataStore.getStateBySiteID(siteID));
                const updatedImageData = {};
                for (const md5Hash of imageMD5s) {
                    if (runtimeStore.getStateBySiteID(siteID).cancelCompression) {
                        break;
                    }
                    serviceContainer.sendIPCEvent(constants_1.IPC_EVENTS.COMPRESS_IMAGE_STARTED, siteID, md5Hash);
                    const currentImageData = siteData.imageData[md5Hash];
                    const { filePath } = currentImageData;
                    if (!fs_extra_1.default.existsSync(filePath)) {
                        const errorMessage = 'File not found!';
                        serviceContainer.sendIPCEvent(constants_1.IPC_EVENTS.COMPRESS_IMAGE_FAIL, siteID, md5Hash, errorMessage);
                        updatedImageData[md5Hash] = mergeImageData(currentImageData, { errorMessage });
                        continue;
                    }
                    /**
                     * We do this step to ensure that image backups are nested under the backup directory in
                     * the same way as they are nested inside wp-conten
                     *
                     * We still include "uploads" in this new file path even though that is the "base" from which we scan
                     * for images so that we can easily expand this to scan other directories within wp-content in the future
                     */
                    let backupPath = filePath.replace(path_1.default.join(site.paths.webRoot, 'wp-content'), backupDirPath);
                    const { dir, name, ext } = path_1.default.parse(backupPath);
                    let deDupeCounter = 1;
                    while (fs_extra_1.default.existsSync(backupPath)) {
                        const tempName = `${name} (${deDupeCounter})${ext}`;
                        backupPath = path_1.default.join(dir, tempName);
                        deDupeCounter++;
                    }
                    fs_extra_1.default.copySync(filePath, backupPath);
                    const args = [];
                    /**
                     * @todo write some tests to cover this case
                     */
                    if (stripMetaData) {
                        args.push('--strip');
                    }
                    /**
                     * the src and dest args are expected as the last
                     * two argv's by jpeg-recompress, so we make sure they are added last
                     */
                    args.push(backupPath, filePath);
                    /**
                     * Wrap this in a promise to ensure that only one image is compressed at a time
                     */
                    yield new Promise((resolve) => {
                        const cp = child_process_1.default.spawn(jpegRecompressBin, args);
                        const stdioCB = (data) => (0, errorReporting_1.reportBinOutput)(filePath, jpegRecompressBin, data);
                        cp.stderr.on('data', stdioCB);
                        cp.stdout.on('data', stdioCB);
                        cp.on('close', (code) => __awaiter(this, void 0, void 0, function* () {
                            if (code !== 0) {
                                const errorMessage = `Failed to process ${filePath}. Exited with code ${code}`;
                                serviceContainer.sendIPCEvent(constants_1.IPC_EVENTS.COMPRESS_IMAGE_FAIL, siteID, md5Hash, errorMessage);
                                updatedImageData[md5Hash] = mergeImageData(currentImageData, { errorMessage });
                                resolve();
                                return;
                            }
                            updatedImageData[md5Hash] = mergeImageData(currentImageData, {
                                compressedImageHash: yield (0, utils_1.getFileHash)(filePath),
                                compressedSize: fs_extra_1.default.statSync(filePath).size,
                            });
                            serviceContainer.sendIPCEvent(constants_1.IPC_EVENTS.COMPRESS_IMAGE_SUCCESS, siteID, updatedImageData[md5Hash]);
                            resolve();
                        }));
                    });
                    imageDataStore.setStateBySiteID(siteID, Object.assign(Object.assign({}, siteData), { imageData: Object.assign(Object.assign({}, siteData.imageData), updatedImageData) }));
                    (0, utils_1.saveImageDataToDisk)(imageDataStore, serviceContainer);
                }
                (0, index_1.updateCancelCompression)(siteID, false);
                serviceContainer.sendIPCEvent(constants_1.IPC_EVENTS.COMPRESS_ALL_IMAGES_COMPLETE, siteID);
                (0, errorReporting_1.reportCompressSuccess)(siteID, imageMD5s.length);
            }
            catch (error) {
                (0, errorReporting_1.reportCompressFailure)(siteID, error);
                /**
                 * @todo add ipc event emitter error here to report errors to the UI
                 */
                return error;
            }
        });
    };
}
exports.compressImagesFactory = compressImagesFactory;
//# sourceMappingURL=compressImages.js.map